/* -*- mode: c++; c-basic-offset: 4; -*- */
#ifndef TBLPGM_FORMAT_HH
#define TBLPGM_FORMAT_HH
#include <string>

namespace table {

    /** %Table program numeric print format class.
      * \brief Format specified.
      * \author John zweiizg <john.zweizig@ligo.org>
      * \version $Id$
      */
    class format_spec {
    public:

	/**  Numeric print type
	 */
	enum num_format {
	    kDefault,  ///< Just print it.
	    kFixed,    ///< Print a fixed point number
	    kSci       ///< Print scientific notation with specified precision
	};
	
	/**  Cell justification.
	 */
	enum fmt_justify {
	    kjNone,
	    kjLeft,
	    kjRight,
	    kjCenter
	};
	
    public:
	/**  Default constructor.
	  *  \brief default constructor.
	  */
	format_spec(void);

	/**  Initializing constructor.
	  *  \brief Initializing constructor.
	  *  \param fmt Format string.
	  */
	format_spec(const std::string& fmt);

	/** Destroy a format specifier.
	  * \brief Destructor.
	  */
	~format_spec(void);

	/**  Define operator() to perform the requested formatting.
	  *  \brief Format a number
	  *  \param num Numeric value to be formatted.
	  *  \return Formatted numeric string.
	  */
	std::string operator()(double num) const;

	/**  Define operator() to perform the requested formatting.
	  *  \brief Format a string
	  *  \param str String to be formatted.
	  *  \return Formatted string.
	  */
	std::string operator()(const std::string& str) const;
	
	/**  Set format specification from a format string.
	  *  \brief Parse format string
	  *  \param fmt Format string.
	  */
	void parse(const std::string& fmt);

	/**  Convert the numeric data to a formatted string.
	  *  \param num Numeric data to be formatted
	  *  \return formatted string. 
	  */
	std::string put_string(double num) const;

	/**  Convert the string data to a formatted string by enforcing the
	  *  width and justification parameters.
	  *  \param str String data to be formatted
	  *  \return Formatted string. 
	  */
	std::string put_string(const std::string& str) const;

	/**  Print numeric data as fixed point numbers.
	  *  \brief Fixed point output
	  *  \param prec Number of digits to the right of the decimal point.
	  */
	void setFixed(int prec);

	/**  Set justification option: "none", "left", "right or "center".
	  *  \brief Field justification.
	  *  \param justify String describing justification.
	  */
	void setJustify(const std::string& justify);

	/**  Print numeric data in c++ default notation.
	  *  \brief Default notation output
	  *  \param prec Number of digits to the right of the decimal point.
	  */
	void setPrecision(int prec);

	/**  Print numeric data in scientific notation.
	  *  \brief Scientific notation output
	  *  \param prec Number of digits to the right of the decimal point.
	  */
	void setScientific(int prec);

	/**  Set the cell width.
	  *  \param wid Width in appropriate units for output device.
	  */
	void setWidth(int wid);

	/**  Return the width of the cell.
	  *  \return width in appropriate units for output device.
	  */
	int width(void) const;

    private:
	num_format  mNumFormat;
	fmt_justify mJustify;
	int         mPrecise;
	int         mWidth;
    };

    //=====================================  Inline functions
    inline int
    format_spec::width(void) const {
        return mWidth;
    }

    inline std::string
    format_spec::operator()(double num) const {
        return put_string(num);
    }

    inline std::string
    format_spec::operator()(const std::string& str) const {
        return put_string(str);
    }

};

#endif // !defined(TBLPGM_FORMAT_HH)
